/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package Celbercor;

import java.io.*;
import java.util.*;
import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import com.sun.crypto.provider.SunJCE;

/** This class is a singleton, with functions and constants that are
 * used by the password and encryption systems.  These functions are
 * factored out into this class to hide some of the messy details,
 * such as the name of the cipher we are using, and how to install the
 * provider that supplies classes for all our encryption stuff.
 *
 * <p> There can be at most one instance of this class per program.
 * To get it, use the static <tt>getInstance()</tt> function:
 *
 * <pre>
 * EncryptionStandard.getInstance().initializeEncryptionLibrary();
 * Cipher c = EncryptionStandard.getInstance().createCipher();
 * </pre> */

public class EncryptionStandard
{

  protected EncryptionStandard()
    {
    }

  /**
   * @return the single instance of this class
   */

  public static EncryptionStandard getInstance()
    {
      if( ourInstance == null )
	{
	  ourInstance = new EncryptionStandard();
	}
      return ourInstance;
    }

  /** Load all needed providers so that further calls to the
   * encryption and security libraries will succeed.  You must call
   * this function before using any encryption functions! */

  public void initializeEncryptionLibrary()
    throws GeneralSecurityException
    {
      Security.addProvider( new SunJCE() );
    }


  /**
   * Creates a <tt>Cipher</tt> object of the proper type.
   */

  public synchronized Cipher createCipher()
    throws GeneralSecurityException
    {
      return Cipher.getInstance( "DES/ECB/PKCS5Padding" );
    }

  /** Generate a random key suitable for use with the standard cipher.
   * This method is used by the <tt>AuthenticationServer</tt> and the
   * <tt>TicketServer</tt> to create session keys.  */

  public synchronized SecretKey createSessionKey()
    throws GeneralSecurityException
    {
      KeyGenerator g = KeyGenerator.getInstance( "DES" );
      g.init( SecureRandom.getInstance( "SHA1PRNG" ) );
      SecretKey k = g.generateKey();
      return k;
    }

  /**
   * Retrieve the key used by the password hashing algorithm.
   */

  public synchronized Key getHashingKey()
    throws GeneralSecurityException
    {
      if( myHashingKey == null )
	{
	  // This is the first few bytes from the SKIP key in sun's
	  // documentation.  We're just going to use this as a DES
	  // key, which is to be used by the hashing algorithm.
	  
	  byte buffer[] = { 
	    (byte)0xF4, (byte)0x88, (byte)0xFD, (byte)0x58,
	    (byte)0x4E, (byte)0x49, (byte)0xDB, (byte)0xCD
	  };
	  
	  DESKeySpec spec = new DESKeySpec( buffer );
	  SecretKeyFactory f = SecretKeyFactory.getInstance( "DES" );
	  myHashingKey = f.generateSecret( spec );
	}

      return myHashingKey;
    }

  /** Convert the string of characters to a secret key.  (Warning:
   * This converts the password to bytes as it works, and may break if
   * you run it in a funky locale.)  You should over-write the
   * password with something to protect it from memory invaders. */

  public synchronized SecretKey convertPassword( char password[] )
    throws GeneralSecurityException
    {
      Key hashingKey = getHashingKey();
      int i;
      byte buffer[] = new byte[8];
      byte hashedPassword[];

      // we have to have at least 8 bytes for DES keys.

      for( i = 0; i < 8; i++ )
	{
	  buffer[i] = (byte)'*';
	}
      
      for( i = 0; i < password.length && i < 8; i++ )
	{
	  buffer[i] = (byte)password[i];
	}

      // Now we hash the password to get a new string of bytes
      Mac hasher = Mac.getInstance( "HmacMD5" );
      hasher.init( hashingKey );
      hashedPassword = hasher.doFinal( buffer );

      // And create a new key
      DESKeySpec spec = new DESKeySpec( hashedPassword );
      SecretKeyFactory f = 
	SecretKeyFactory.getInstance( "DES" );
      
      SecretKey key = f.generateSecret( spec );

      return key;
    }

  private static char trans[] = { '0', '1', '2', '3', '4', '5',
				  '6', '7', '8', '9',
				  'A', 'B', 'C', 'D', 'E', 'F' };

  /** Print a byte array to <tt>System.out</tt>, from lowest order
   * nibble to highest order.  */

  public static void displayByteArray( byte array[] )
    {
      int i;
      byte b;

      for( i = 0; i < array.length; i++ )
	{
	  b = array[i];
	  System.out.print( trans[b & 0xF] );
	  System.out.print( trans[(b >> 4) & 0xF] );
	}
    }

  /** Read a user password from given input stream. (This is from
   * Sun's documentation.  The idea is to read it into a
   * <tt>char[]</tt> rather than a <tt>String</tt> so you can write
   * over it with zeros once you're done.  The password still appears
   * on the screen as you type because java has no console control.)
   * */

  public char[] readPassword(InputStream in) 
    throws IOException
    {
      char[] lineBuffer;
      char[] buf;
      int i;
    
      buf = lineBuffer = new char[128];

      int room = buf.length;
      int offset = 0;
      int c;

    loop:   
      while (true)
	{
	  switch (c = in.read())
	    {
	    case -1: 
	    case '\n':
	      break loop;

	    case '\r':
	      int c2 = in.read();
	      if ((c2 != '\n') && (c2 != -1))
		{
		  if (!(in instanceof PushbackInputStream))
		    {
		      in = new PushbackInputStream(in);
		    }
		  ((PushbackInputStream)in).unread(c2);
		}
	      else 
		{
		  break loop;
		}

	    default:
	      if (--room < 0)
		{
		  buf = new char[offset + 128];
		  room = buf.length - offset - 1;
		  System.arraycopy(lineBuffer, 0, buf, 0, offset);
		  Arrays.fill(lineBuffer, ' ');
		  lineBuffer = buf;
		}
	      buf[offset++] = (char) c;
	      break;
	    }
	}
      
      if (offset == 0)
	{
	  return null;
	}


      char[] ret = new char[offset];
      System.arraycopy(buf, 0, ret, 0, offset);
      Arrays.fill(buf, ' ');

      return ret;
    }

  private Key myHashingKey;
  private static EncryptionStandard ourInstance;

}
