/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package Celbercor;

import java.io.*;
import java.util.*;
import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;

/** This class represents a mapping from user and service names to
 * encryption keys that are based upon passwords; its main function
 * allows you to manage password files.  The passwords themselves are
 * never needed by the Celbercor authentication protocol and are not
 * stored in this object for security reasons: If a server were to
 * crash and dump core, or someone were to steal the password file,
 * they might be able to wade through the data and find the passwords.
 * Since only the keys are stored, this is not a problem. This class
 * is serializable, and the servers will read an object of this class
 * from a file when they start up.
 *
 * <p> Given a password, the encryption key is determined by hashing
 * the password with a message authentication code (MAC).  The
 * resulting checksum is used to generate a secret encryption key.
 *
 * <p> This class's <tt>main()</tt> function allows you to create and
 * add users to a password file, which can be used by the servers. */

public class PasswordKeyList
  implements Serializable
{
  /** Set the key for a user or service name from the given password
   * data.  */

  public void set( String name, char password[] )
    throws GeneralSecurityException
    {
      set( name, 
	   EncryptionStandard.getInstance().convertPassword( password ) );
    }

  /**
   * Set the key for a particular user or service name.
   */

  public synchronized void set( String name, SecretKey key )
    {
      myMap.put( name, key );
    }

  /**
   * Get the key for a particular user or service name.
   */

  public synchronized SecretKey get( String name )
    {
      return (SecretKey)myMap.get( name );
    }

  /**
   * Read password data from a file.
   *
   * @return a <tt>PasswordList</tt> if succeeded, <tt>null</tt> if
   * failed.  */

  public static PasswordKeyList readPasswords( String fileName )
    {
      try
	{
	  InputStream fin = new FileInputStream( fileName );
	  ObjectInput in = new ObjectInputStream( fin );
	  return (PasswordKeyList)in.readObject();
	}
      catch( Exception ex )
	{
	  ex.printStackTrace();
	}
      return null;
    }

  /**
   * Write this object to a file.
   */

  public void writePasswords( String fileName )
    throws IOException
    {
      OutputStream fout = new FileOutputStream( fileName );
      ObjectOutput out = new ObjectOutputStream( fout );
      out.writeObject( this );
    }


  private Hashtable myMap = new Hashtable();

  /** This is a short program that reads a <tt>PasswordKeyList</tt>
   * from a file, adds a password to it, and writes it back.  It takes
   * arguments of
   *
   * <pre>
   * java Celbercor.PasswordKeyList <i>fileName</i> <i>userName</i>
   * </pre>
   *
   * It will load a <tt>PasswordKeyList</tt> object from the give
   * file, prompt for a password, and add the resulting key to the
   * table under <i>userName</i>.  If you specify <tt>new</tt> as the
   * user name, it will create an empty password file for you.  (You
   * have to create an empty file before you can add password keys to
   * it.) */

  public static void main( String args[] )
    {
      String fileName;
      String name;
      char buffer[];
      PasswordKeyList pkl;

      if( args.length != 2 )
	{
	  System.out.println( "You must specify: fileName userName" );
	  System.exit(1);
	}

      Provider p = new com.sun.crypto.provider.SunJCE();
      Security.addProvider( p );

      fileName = args[0];
      name = args[1];

      if( name.equals( "new" ) )
	{
	  System.out.println( "Creating new list" );
	  pkl = new PasswordKeyList();
	}
      else
	{
	  // Editing existing

	  pkl = readPasswords( fileName );
	  if( pkl == null )
	    {
	      System.out.println( "Unable to read passwords from " 
				  + fileName );
	      System.exit(2);
	    }

	  System.out.println( "Enter password: " );
	  System.out.flush();
	  try
	    {
	      buffer = 
		EncryptionStandard.getInstance().readPassword( System.in );
	      pkl.set( name, buffer );
	    }
	  catch( IOException ex )
	    {
	      System.out.println( "Unable to input password:\n" + ex );
	      System.exit(3);
	    }
	  catch( GeneralSecurityException ex )
	    {
	      System.out.println( "Unable to generate key:\n"
				  + ex );
	      System.exit(4);
	    }
	}

      try
	{
	  pkl.writePasswords( fileName );
	}
      catch( IOException ex )
	{
	  System.out.println( "Unable to save passwords:\n" + ex );
	  System.exit(4);
	}
    }
}

