/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package ChatRoom.Client;

import ChatRoom.Tools.*;
import java.io.*;
import java.net.*;
import ChatRoom.Tools.Reader;
import ChatRoom.Tools.Writer;

/** This class encapsulates the main pipeline of a client.
 *
 * <pre>
 * Reader ==> Handler ==> User ==> Writer
 * </pre>
 *
 * Its most important method is <tt>run()</tt>, which runs an infinite
 * loop that reads messages from a <tt>Reader</tt> and sends them to a
 * <tt>MessageHandler</tt>.  The idea here is that you can plug in
 * different kinds of readers, writers, and handlers to make the
 * client do different things.  For example, the monitor client uses a
 * <tt>DataInputReader</tt> connected to a TCP/IP socket, and a
 * <tt>MessagePrinter</tt> handler that just prints messages out to
 * the screen.  The GUI client that talks to a server uses the same
 * kind of reader, but its handler puts messages into a
 * <tt>TextArea</tt> widget.  The GUI client that uses multicast uses
 * the same kind of handler, but uses a <tt>DatagramReader</tt>
 * instead of a <tt>DataInputReader</tt>.
 *
 * <p> One interesting thing that you could do is set up a forwarding
 * system that receives messages from one chat room and forwards them
 * to another.  By using different reader and writer classes, it could
 * connect a multicast chat room to a server-based chat room and
 * vice-versa.
 *
 * <p> You might also consider writing a writer class that sends
 * messages to a list of other writers, which would allow you to send
 * the same message to multiple chat rooms.  Likewise, you could write
 * a reader class that can receive messages from multiple chat rooms.
 *
 * <p> (<i>Note:</i> You can also get really crazy and re-write the
 * <tt>ThreadedServer</tt> to use instances of this class, but I
 * thought that was a little too confusing, so I didn't do it.)
 *
 * <p> To use this <tt>Client</tt> class, connect it to a
 * <tt>Reader</tt> and a <tt>Writer</tt>, plug in a
 * <tt>MessageHandler</tt>, and call <tt>run()</tt>.  */

public class Client
  implements Runnable
{
  /** Construct a client that reads messages from the given reader,
   * and writes them to the given writer. */

  public Client( Reader reader, Writer writer )
    {
      myReader = reader;
      myWriter = writer;
    }

  /** Specify the handler.  Its handleMessage() method is called on
   * each incoming message. */

  public synchronized void setHandler( MessageHandler mh )
    {
      myHandler = mh;
    }

  /** Do the work of reading messages and passing them to the handler.
   * Doesn't return until the reader runs out of messages and returns
   * null. */

  public void run()
    {
      Message m = myReader.getNextMessage();
      while( m != null )
	{
	  if( myHandler != null )
	    {
	      myHandler.handleMessage( m );
	    }
	  m = myReader.getNextMessage();
	}
    }

  /**
   * Send a message down this client's writer.
   */

  public void sendMessage( Message m )
    throws IOException
    {
      myWriter.deliverMessage( m );
    }

  /** Create a socket and contact a chat room server at the given host
   * and IP port.  Throw an exception if this is not possible. */

  public static Client createViaInternet( String host, int port )
    throws UnknownHostException, IOException
    {
      Socket s = new Socket( host, port );
      DataInputStream in = new DataInputStream( s.getInputStream() );
      Reader r = new DataInputReader( in );
      DataOutputStream out = new DataOutputStream( s.getOutputStream() );
      Writer w = new DataOutputWriter( out );
      return new Client( r, w );
    }


  /** Create a multicast socket with the given multicast address.
   * This kind of client doesn't use a server, since the router will
   * do all the work.
   *
   * @param packetSize Maximum packet size to send.
   *
   * @param a Address to use (must be class D)
   *
   * @param port Port to use.
   *
   * @param ttl Time-to-live constant.  
   * This tells how far chat messages will go.  According to 
   * <a href="http://ganges.cs.tcd.ie/4ba2/multicast/antony/index.html">this multicast tutorial,</a> these are some values for that constant:
<dl>
<dt><b>0</b>
<dd>restricted to the same host
<dt><b>1</b>
<dd>restricted to the same subnet
<dt><b>32</b>
<dd>restricted to the same site
<dt><b>64</b>
<dd>restricted to the same region
<dt><b>128</b>
<dd>restricted to the same continent
<dt><b>255</b>
<dd>unrestricted
</dl>
   **/

  public static Client createViaMulticast(  int packetSize, InetAddress a, 
					    int port, int ttl )
    throws IOException
    {
      MulticastSocket s = new MulticastSocket(port);
      s.setTimeToLive( ttl );
      s.joinGroup( a );
      Reader r = new DatagramReader( s, a, port, packetSize );
      Writer w = new DatagramWriter( s, a, port, packetSize );
      return new Client( r, w );
    }
					   

  private Reader myReader;
  private Writer myWriter;
  private MessageHandler myHandler;
}
