/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package ChatRoom.FlamingSword;

import java.io.*;
import java.net.*;
import java.util.*;
import java.security.*;
import javax.crypto.*;
import ChatRoom.Security.*;
import ChatRoom.Tools.*;
import ChatRoom.Tools.Reader;
import ChatRoom.Tools.Writer;
import ChatRoom.Tools.Message;
import FlamingSword.*;

/** This class does authentication for chat room clients using the
 * Flaming Sword protocol.  To use this class, create an instance of
 * it, call <tt>initialize()</tt>, and use the return value of
 * <tt>sendVerification()</tt> to construct a <tt>Client</tt> object.
 * Much of the dirty work has been factored out into
 * <tt>SecureClient.createViaInternet()</tt>.
 *
 * @see ChatRoom.Security.SecureClient */

public class ClientAuthenticationImpl
  extends AgentBase
  implements ClientAuthentication
{
  /**
   * @param endClient the name of this client
   *
   * @param endService the name of the service we are trying to contact
   *
   * @param fileName abbreviated name of files where the server's
   * public and private keys are held 
   *
   * @param authFileName abbreviated name of the file where the
   * authentication server's public key is held. 
   *
   * @param authHost host where the authentication server is running
   *
   * @param authPort port where the authentication server is listening
   */

  public ClientAuthenticationImpl( String endClient, String endService,
				   String fileName,
				   String authFileName,
				   String authHost, int authPort )
    {
      myEndClient = endClient;
      myEndService = endService;
      myFileName = fileName;
      myAuthFileName = authFileName;
      myAuthHost = authHost;
      myAuthPort = authPort;
    }

  /* <ul>
   * <li> <tt>flamingsword.authentication.host</tt>.  
   * (<i>Default value: <tt>AuthenticationServer.DEFAULT_HOST</tt></i>)
   * <li> <tt>flamingsword.authentication.port</tt>.
   * (<i>Default value: <tt>AuthenticationServer.DEFAULT_PORT</tt></i>)
   * </ul>
   */

  /** Initializes this authentication object.  The service name is set
   * to the chat room.  The key file name is set to the client name,
   * that is, if you log in as "freddy" it will look in "freddy.pub"
   * and "freddy.pri" for public and private keys.  The host name and
   * port of the authentication server are taken from the <a
   * href="../../FlamingSword/package-summary.html#Properties">system
   * properties</a> (using a default value if the property is not set)
   *
   *
   * @param endClient the name of this client
   *
   * @param authFileName abbreviated name of the file where the
   * authentication server's public key is held. 
   * */
  public ClientAuthenticationImpl( String endClient,
				   String authFileName )
    {
      this( endClient, Constants.CHATROOM_NAME, endClient, authFileName,
	    System.getProperty( "flamingsword.authentication.host",
				AuthenticationServer.DEFAULT_HOST ),
	    (Integer.getInteger( "flamingsword.authentication.port",
				AuthenticationServer.DEFAULT_PORT ))
	    .intValue());
	    
    }

  public boolean initialize()
    {
      try
	{
	  initialize( myFileName, myAuthFileName );
	  
	}
      catch( Exception ex )
	{
	  ex.printStackTrace();
	  return false;
	}
      return true;
    }

  public RWPair sendVerification( String host, int port )
    throws IOException, GeneralSecurityException, ClassNotFoundException
    {
      AuthenticationReplyMessage reply = null;

      reply =
	Utilities.contactAuthenticationServer( myEndClient, myEndService,
					       myAuthHost, myAuthPort,
					       getAuthPublicKey(),
					       getPrivateKey() );
      if( reply == null )
	{
	  System.out.println( "Reply was null" );
	  return null;
	}

      // Now have to contact the end service...
      System.out.println( "Contacting chat room " 
			  + host + ':' + port );

      Ticket t = reply.getServiceTicket();

      ServiceInfo reply2 =
	Utilities.contactService( reply.getSealedClientTicket(),
				  t.getNonce(),
				  host, port,
				  getPrivateKey(), t.getPublicKey() );

      if( reply2 == null )
	{
	  System.out.println( "Reply was null" );
	  return null;
	}

      Cipher serviceCipher;
      Cipher clientCipher;

      serviceCipher = 
	EncryptionStandard.getInstance().createCipher();
      serviceCipher.init( Cipher.DECRYPT_MODE, t.getPublicKey() );
      
      clientCipher = 
	EncryptionStandard.getInstance().createCipher();
      clientCipher.init( Cipher.ENCRYPT_MODE, getPrivateKey() );

      Reader r = new SignedSealedObjectReader( serviceCipher, 
					       reply2.getInput(), 
					       t.getNonce() );
      Writer w = new SignedSealedObjectWriter( clientCipher, 
					       reply2.getOutput(), 
					       t.getNonce() );
      return new RWPair( r, w, null );

    }

  private String myEndClient;
  private String myEndService;
  private String myFileName;
  private String myAuthFileName;  
  private String myAuthHost;
  private int myAuthPort;
}

