/*
This file is part of ChatRoom.

Copyright (C) 1999 William Garrett Mitchener
E-mail: wmitchen@princeton.edu

ChatRoom is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

ChatRoom is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with ChatRoom; if not, write to the Free Software Foundation,
Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/
package ChatRoom.Security;

import ChatRoom.Tools.*;
import ChatRoom.Server.*;
import java.io.*;
import java.net.*;
import java.util.*;
import ChatRoom.Tools.Reader;
import ChatRoom.Tools.Writer;

/** This class is a multi-threaded secure server for a chat room.  It
 * runs an infinite loop where it waits for clients to call it up,
 * then spawns a new thread running a <tt>MessageConsumer</tt> to
 * handle the client. */

public class SecureServer
  implements Runnable
{
  /** Initialize a <tt>SecureServer</tt> to use the given
   * authentication object.  You must call <tt>initialize()</tt> on
   * <tt>auth</tt> before passing it to this constructor.
   *
   * @param auth an initialized authentication object. */

  public SecureServer( ServerAuthentication auth )
    {
      myAuthenticator = auth;
    }

  /** Set this server to listen for connections on the given port.  */

  public void open( int port )
    throws IOException
    {
      myServerSocket = new ServerSocket( port );
    }

  /** Wait for a client to contact the server, then return a socket
   * connecting to it.  This will throw a
   * <tt>NullPointerException</tt> unless you have successfully called
   * <tt>open()</tt> on this server.
   *
   * @see #open( int ) */

  public Socket getNextClient()
    throws IOException
    {
      return myServerSocket.accept();
    }

  /** Run the the main loop.  Wait for clients, and start a new thread
   * to handle each one.  This will throw a
   * <tt>NullPointerException</tt> unless you have successfully called
   * <tt>open()</tt> on this server.
   *
   * @see #open( int )*/

  public void run()
    {
      Socket s;
      Thread t;
      Reader r;
      Writer w;
      MessageConsumer mc;

      while( true )
	{
	  try
	    {
	      s = getNextClient();
	      System.out.println( "Contact established" );

	      RWPair p = myAuthenticator.verifyClient( s );
	      if( p == null )
		{
		  System.out.println( "Authentication failed" );
		}
	      else
		{
		  System.out.println( "Authentication suceeded" );

	      
		  // Give the server a way to send messages to the new client

		  w = p.getWriter();
		  myClientList.addClient( w );

		  r = p.getReader();
		  mc = new MessageConsumer( myClientList, r );
		  t = new Thread( mc );
		  t.start();
		  System.out.println( "New thread started for " + p.getName() );

		  // Tell everyone someone new has arrived
		  System.out.println( "Delivering login message for " + p.getName() );
		  myClientList.deliverMessage(
		      new Message( "server", 
				   p.getName() + " just logged in" ));
					     		}
	    }
	  catch( IOException ex )
	    {
	      System.out.println( "Unexpected exception:" );
	      ex.printStackTrace();
	    }
	}
    }

  private ServerSocket myServerSocket;
  private ClientList myClientList = new ClientList();
  private ServerAuthentication myAuthenticator;
}

