package FlamingSword;

import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;
import javax.crypto.spec.*;

/** This is the public key server for the Flaming Sword protocol.  It
 * has a <tt>main()</tt> function which will actually run the
 * server. */

public class AuthenticationServer
{

  /**
   * Default port number to listen on (10305).
   */

  public static final int DEFAULT_PORT = 10305;

  /** Default abbreviated file name to look for keys in
   * ("authserver").  */

  //  public static final String DEFAULT_FILE_NAME = "authserver";

  /** The default host the authentication server runs on.  (Current
   * value: the local host) */

  public static final String DEFAULT_HOST;


  static
    {
      InetAddress a = null;
      try
	{
	  a = InetAddress.getLocalHost();
	}
      catch( UnknownHostException ex )
	{
	  ex.printStackTrace();
	}
      
      if( a == null )
	{
	  DEFAULT_HOST = "localhost";
	}
      else
	{
	  DEFAULT_HOST = a.getHostName();
	}
    }

  /**
   * Initialize this server.
   *
   * @param keyFileName abbreviated name of the key file for this
   * server.  The public key should be in
   * <tt><i>keyFileName</i>.pub</tt> and the private key should be in
   * <tt><i>keyFileName</i>.pri</tt>.
   *
   * @param port the port number to listen to  */

  public void initialize( String keyFileName, int port )
    throws IOException, GeneralSecurityException, ClassNotFoundException
    {
      myPort = port;
      mySocket = new ServerSocket( port );
      ObjectInput in;
      FileInputStream priIn = new FileInputStream( keyFileName + ".pri" );
      in = new ObjectInputStream( priIn );
      PrivateKey priKey = (PrivateKey)in.readObject();
      myPrivateCipher = 
	EncryptionStandard.getInstance().createCipher();
      myPrivateCipher.init( Cipher.ENCRYPT_MODE, priKey );
    }

  /** Read a list of passwords from the given file.  This replaces any
   * existing passwords this server was using.  */

  public void readPublicKeys( String fileName )
    throws IOException, ClassNotFoundException
    {
      myKeys = PublicKeyList.read( fileName );
    }


  /** Handle one client.  Listen for a request and send a reply.  (The
   * reply will be <tt>null</tt> if the request was invalid.)  */

  public void handleOneConnection( Socket s )
    {
      try
	{
	  ObjectOutput out = new ObjectOutputStream( s.getOutputStream() );
	  ObjectInput in = new ObjectInputStream( s.getInputStream() );
	  AuthenticationRequestMessage request = null;
	  AuthenticationReplyMessage reply = null;
	  SealedObject sealedReply = null;


	  System.out.println( "Contact established, reading request" );
	  request = (AuthenticationRequestMessage)in.readObject();

	  PublicKey serviceKey = myKeys.get( request.getEndService() );
	  PublicKey clientKey = myKeys.get( request.getEndClient() );

	  if( serviceKey == null )
	    {
	      System.out.println( "Unknown service: " 
				  + request.getEndService() );
	    }
	  else if( clientKey == null )
	    {
	      System.out.println( "Unknown client: "
				  + request.getEndClient() );
	    }
	  else
	    {
	      System.out.println( "Formulating reply" );
	      Object nonce = new RandomByteArray();
	      long time = System.currentTimeMillis();
	      
	      Ticket clientTicket = 
		new Ticket( request.getEndClient(), clientKey,
			    nonce, time );
	      
	      Ticket serviceTicket = 
		new Ticket( request.getEndService(), serviceKey,
			    nonce, time );
	      
	      reply = new AuthenticationReplyMessage( 
	          new SealedObject( clientTicket, myPrivateCipher ),
		  new SealedObject( serviceTicket, myPrivateCipher ) );
	      
	      Cipher clientCipher = 
		EncryptionStandard.getInstance().createCipher();
	      clientCipher.init( Cipher.ENCRYPT_MODE, clientKey );
	      
	      sealedReply = new SealedObject( reply, clientCipher );
	      
	      System.out.println( "Sending reply" );
	    }
	  out.writeObject( sealedReply );
	}
      catch( IOException ex )
	{
	  System.out.println( "Connection to client failed" );
	  ex.printStackTrace();
	}
      catch( GeneralSecurityException ex )
	{
	  System.out.println( "Connection to client failed" );
	  ex.printStackTrace();
	}
      catch( ClassNotFoundException ex )
	{
	  System.out.println( "Connection to client failed" );
	  ex.printStackTrace();
	}
    }

  /** Run an infinite loop of <tt>handleOneConnection()</tt>.  */

  public void run()
    {
      while( true )
	{
	  try
	    {
	      handleOneConnection( mySocket.accept() );
	    }
	  catch( IOException ex )
	    {
	      System.out.println( "Server socket failed:" );
	      ex.printStackTrace();
	    }
	}
    }


  /**
   * @return the port this server is listening to
   */

  public int getPort()
    {
      return myPort;
    }

  private int myPort;
  private ServerSocket mySocket;
  private PublicKeyList myKeys;
  private Cipher myPrivateCipher;

  /**
   * Start an authentication server.  The arguments are:
   *
   * <pre>
   * java Celbercor.AuthenticationServer <i>myKeys</i> <i>publicKeyFile</i> [<i>port</i>]
   * </pre>
   *
   * where <i>myKeys</i> is the abbreviated file name for the server's
   * public and private keys (<tt>.pub</tt> and <tt>.pri</tt> will be
   * appended to this string), <tt><i>publicKeyFile</i></tt> is the
   * full name of a file containing the public key list, and
   * <i>port</i> is the port to listen to (default value is
   * <tt>AuthenticationServer.DEFAULT_PORT</tt>).  */

  public static void main( String args[] )
    {
      String fileName = null;
      String publicKeyFile = null;
      int port = DEFAULT_PORT;
      AuthenticationServer server = new AuthenticationServer();

      if( args.length < 2 )
	{
	  System.out.println( "Arguments: myKeys publicKeyFile [port]" );
	  System.exit(1);
	}

      fileName = args[0];
      publicKeyFile = args[1];

      if( args.length >= 3 )
	{
	  port = Integer.parseInt( args[1] );
	}

      System.out.println( "Initializing server, my keys: " + fileName
			  + " public key file: " + publicKeyFile
			  + " port:"  + port );

      try
	{
	  EncryptionStandard.getInstance().initializeEncryptionLibrary();
	  server.initialize( fileName, port );
	  server.readPublicKeys( publicKeyFile );
	}
      catch( FileNotFoundException ex )
	{
	  System.out.println( ex );
	  System.exit(1);
	}
      catch( Exception ex )
	{
	  ex.printStackTrace();
	  System.exit(1);
	}

      System.out.println( "Listening for clients" );
      server.run();
      
    }

}
