package FlamingSword;

import java.io.*;
import java.util.*;
import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;

/** This class represents a mapping from user and service names to
 * public keys; its <tt>main()</tt> function allows simple
 * manipulation of a file containing a public key list. */

public class PublicKeyList
  implements Serializable
{

  /**
   * Set the key that goes with the given user or service name.
   */

  public synchronized void set( String name, PublicKey key )
    {
      myMap.put( name, key );
    }

  /**
   * Get the key for a particular user or service name.
   */

  public synchronized PublicKey get( String name )
    {
      return (PublicKey)myMap.get( name );
    }

  /**
   * Read public key data from a file.
   *
   * @return a <tt>PublicKeyList</tt> if succeeded, throws an
   * exception if failed.  */

  public static PublicKeyList read( String fileName )
    throws IOException, ClassNotFoundException
    {
      InputStream fin = new FileInputStream( fileName );
      ObjectInput in = new ObjectInputStream( fin );
      return (PublicKeyList)in.readObject();
    }

  /**
   * Write this object to a file.
   */

  public void write( String fileName )
    throws IOException
    {
      OutputStream fout = new FileOutputStream( fileName );
      ObjectOutput out = new ObjectOutputStream( fout );
      out.writeObject( this );
    }


  private Hashtable myMap = new Hashtable();

  /** This is a short program that reads a <tt>PublicKeyList</tt> from
   * a file, generates a new key pair, adds the public key to the
   * list, and writes it back.  It takes arguments of:
   *
   * <pre>
   * java FlamingSword.PublicKeyList <i>keyFileName</i> <i>userName</i>
   * </pre>
   * 
   * If <i>userName</i> is <tt>new</tt>, the program will create a
   * new, empty list.  (This must be done before any keys can be added
   * to it.)  If <i>userName</i> is something other than <tt>new</tt>,
   * the program will generate new public and private keys and store
   * the public key in the key file.  It also writes the public key to
   * a file named <tt><i>userName</i>.pub</tt> and the private key to
   * a file named <tt><i>userName</i>.pri</tt> so they can be used by
   * other programs. */

  public static void main( String args[] )
    {
      String fileName;
      String userName;
      char buffer[];
      PublicKeyList pkl = null;

      if( args.length != 2 )
	{
	  System.out.println( "You must specify: fileName userName" );
	  System.exit(1);
	}

      fileName = args[0];
      userName = args[1];

      try
	{
	  EncryptionStandard.getInstance().initializeEncryptionLibrary();
	}
      catch( GeneralSecurityException ex )
	{
	  System.out.println( "Unable to initialize encryption library:" );
	  ex.printStackTrace();
	}

      if( userName.equals( "new" ) )
	{
	  System.out.println( "Creating new list" );
	  pkl = new PublicKeyList();
	}
      else
	{
	  // Editing existing
	  
	  try
	    {
	      pkl = read( fileName );
	    }
	  catch( IOException ex )
	    {
	      System.out.println( "Unable to read " 
				  + fileName );
	      System.exit(2);
	    }
	  catch( ClassNotFoundException ex )
	    {
	      System.out.println( "Unable to deserialize " 
				  + fileName );
	      ex.printStackTrace();
	      System.exit(2);
	    }

	  try
	    {
	      System.out.println( "Generating new key pair" );
	      KeyPair p = EncryptionStandard.getInstance().generateKeyPair();
	      System.out.println( "Saving keys" );
	      pkl.set( userName, p.getPublic() );

	      ObjectOutput out;
	      OutputStream pubOut = 
		new FileOutputStream( userName + ".pub" );
	      out = new ObjectOutputStream( pubOut );
	      out.writeObject( p.getPublic() );
	      OutputStream priOut =
		new FileOutputStream( userName + ".pri" );
	      out = new ObjectOutputStream( priOut );
	      out.writeObject( p.getPrivate() );
	    }
	  catch( Exception ex )
	    {
	      ex.printStackTrace();
	      System.exit(3);
	    }
	}

      try
	{
	  pkl.write( fileName );
	}
      catch( IOException ex )
	{
	  System.out.println( "Unable to save passwords:" );
	  ex.printStackTrace();
	  System.exit(4);
	}
    }
}

